﻿(function () {
    'use strict';

    var jobListViewComponent = {
        templateUrl: Umbraco.Sys.ServerVariables.application.applicationPath + 'App_Plugins/TranslationManager/jobs/joblistview.html',
        bindings: {
            selectable: '<',
            statusRange: '<',
            culture: '<',
            onRefresh: '&'
        },
        controllerAs: 'vm',
        controller: jobListViewController
    };

    function jobListViewController($scope, $attrs, editorService,
        notificationsService,
        translateJobService) {

        var vm = this;
        vm.page = 1;
        vm.loading = true;
        vm.results = [];

        vm.viewJob = viewJob;
        vm.refresh = refresh;

        vm.next = next;
        vm.prev = prev;
        vm.gotoPage = gotoPage;

        $scope.$on('translate-reloaded', function () {
            vm.refresh();
        });

        vm.$onInit = function () {
            vm.refresh();
        };

        ////////////////
        function loadJobs(culture, status, page) {
            vm.loading = true;

            translateJobService.getByCultureAndStatusPaged(culture, status[0], status[1], page)
                .then(function (result) {
                    vm.results = result.data;
                    vm.loading = false;
                }, function (error) {
                    notificationsService
                        .error('Load failed', error.data.ExceptionMessage);
                });
        }


        function refresh() {
            if ($attrs.onRefresh !== undefined) {
                vm.loading = true;
                vm.onRefresh({
                    page: vm.page, cb: function (results) {
                        vm.results = results;
                        vm.loading = false;
                    }
                });
            }
            else {
                loadJobs(vm.culture, vm.statusRange, vm.page);
            }
        }

        function viewJob(jobId) {
            /*
            editorService.open({
                jobId: jobId,
                title: 'Job View',
                view: Umbraco.Sys.ServerVariables.translationManager.Plugin + 'backoffice/jobs/view.html',
                submit: function () {
                    editorService.close();
                },
                close: function () {
                    refresh();
                    editorService.close();
                }
            });*/

            window.location.href = '#/translation/jobs/view/' + jobId;
        }

        //////// paging
        function next() {
            vm.page++;
            refresh();
        }

        function prev() {
            vm.page--;
            refresh();
        }

        function gotoPage(pageNo) {
            vm.page = pageNo;
            refresh();
        }
    }

    angular.module('umbraco')
        .component('translateJobListView', jobListViewComponent);
})();